package de.ozgcloud.xta.client;

import de.ozgcloud.xta.client.config.XtaClientConfig;
import de.ozgcloud.xta.client.config.XtaConfigValidator;
import de.ozgcloud.xta.client.core.XtaClientServiceFactory;
import de.ozgcloud.xta.client.core.XtaExceptionHandlerFactory;
import de.ozgcloud.xta.client.exception.XtaClientInitializationException;
import lombok.Builder;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Builder
public class XtaClientFactory {

	private final XtaConfigValidator configValidator;
	private final XtaClientServiceFactory xtaClientServiceFactory;
	private final XtaClientConfig config;
	private final XtaExceptionHandlerFactory exceptionHandlerFactory;

	public static XtaClientFactory from(final XtaClientConfig config) {
		return XtaClientFactory.builder()
				.config(config)
				.configValidator(XtaConfigValidator.builder().build())
				.xtaClientServiceFactory(XtaClientServiceFactory.from(config))
				.exceptionHandlerFactory(XtaExceptionHandlerFactory.builder().build())
				.build();
	}

	public XtaClient create() throws XtaClientInitializationException {
		configValidator.validate(config);
		return XtaClient.builder()
				.config(config)
				.service(xtaClientServiceFactory.create())
				.exceptionHandler(exceptionHandlerFactory.create())
				.build();
	}
}
