package de.ozgcloud.xta.client.config;

import static lombok.AccessLevel.*;

import java.util.stream.Collectors;

import jakarta.validation.Validation;
import jakarta.validation.Validator;

import de.ozgcloud.xta.client.exception.XtaClientInitializationException;
import lombok.Builder;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = PRIVATE)
@Builder
public class XtaConfigValidator {
	private static final Validator VALIDATOR;

	static {
		try (var factory = Validation.buildDefaultValidatorFactory()) {
			VALIDATOR = factory.getValidator();
		}
	}

	public void validate(final XtaClientConfig config) throws XtaClientInitializationException {
		var violations = VALIDATOR.validate(config);
		if (!violations.isEmpty()) {
			throw new XtaClientInitializationException("Client configuration is invalid:\n" + violations.stream()
					.map(v -> "'%s' %s".formatted(v.getPropertyPath().toString(), v.getMessage()))
					.collect(Collectors.joining("\n")));
		}
	}
}
