package de.ozgcloud.xta.client.core;

import org.mapstruct.factory.Mappers;

import de.ozgcloud.xta.client.config.XtaClientConfig;
import de.ozgcloud.xta.client.exception.XtaClientInitializationException;
import de.ozgcloud.xta.client.mapper.RequestMapper;
import de.ozgcloud.xta.client.mapper.ResponseMapper;
import lombok.Builder;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Builder
public class WrappedXtaServiceFactory {

	private final XtaPortTripleFactory xtaPortTripleFactory;
	private final RequestMapper requestMapper;
	private final ResponseMapper responseMapper;

	public static WrappedXtaServiceFactory from(final XtaClientConfig config) {
		return WrappedXtaServiceFactory.builder()
				.xtaPortTripleFactory(XtaPortTripleFactory.from(config))
				.requestMapper(Mappers.getMapper(RequestMapper.class))
				.responseMapper(Mappers.getMapper(ResponseMapper.class))
				.build();
	}

	public WrappedXtaService create() throws XtaClientInitializationException {
		return WrappedXtaService.builder()
				.ports(xtaPortTripleFactory.create())
				.requestMapper(requestMapper)
				.responseMapper(responseMapper)
				.build();
	}
}
