package de.ozgcloud.xta.client.model;

import jakarta.annotation.Nullable;
import jakarta.validation.constraints.NotBlank;

import lombok.Builder;

/**
 * An XTA party identifier which identifies XTA (reader/author) clients or (sender/receiver) server.
 *
 * <p>
 * This client library only uses client identifiers and assumes an identifier type of {@code xoev}. The client certificate authorizes the use of an
 * identifier value as reader/author. Note that name and category may be freely chosen and serve as documentation of the actual value.
 * </p>
 *
 * Example:
 * <pre>
 * var clientIdentifier = XtaIdentifier.builder()
 *  .value("gae:dev-environment@ozg-cloud.de")
 *  .category("Generischer Antragsempfänger")
 *  .name("OZG-Cloud Dev")
 *  .build();
 * </pre>
 *
 * @param name     Name of the identifier. <p>This field is optional.</p>
 * @param category Category of the identifier. <p>This field is optional.</p>
 * @param value    Value of the identifier. <p>This field is required.</p>
 */
@Builder
public record XtaIdentifier(
		@Nullable String name,
		@Nullable String category,
		@NotBlank String value
) {
}
