package de.ozgcloud.xta.client.model;

import java.util.List;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;

import lombok.Builder;

/**
 * An XTA message consisting of metadata, a message file and optional attachment files.
 *
 * @param metaData Metadata of the message <p>This field is required.</p>
 * @param messageFile Main message of the message <p>This field is required.</p>
 * @param attachmentFiles Attachment files of the message <p>Note that this list should always be empty for xdomea messages.</p>
 */
@Builder(toBuilder = true)
public record XtaMessage(
		@NotNull @Valid XtaMessageMetaData metaData,
		@NotNull @Valid XtaFile messageFile,
		@NotNull List<@Valid XtaFile> attachmentFiles
) {
}
