package de.ozgcloud.xta.client.xdomea.mapper;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.xta.client.exception.XtaClientRuntimeException;
import de.ozgcloud.xta.client.model.XtaIdentifier;
import de.ozgcloud.xta.client.model.XtaMessageMetaData;
import de.ozgcloud.xta.client.xdomea.reader.XdomeaXmlValues;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR
)
public interface MetadataMapper {
	String CODE_0201 = "0201";
	String CODE_0401 = "0401";
	String CODE_0402 = "0402";
	String MESSAGE_TYPE_CODE_0201 = "Geschaeftsgang.Geschaeftsgang.0201";
	String MESSAGE_TYPE_CODE_0401 = "Abgabe.Abgabe.0401";
	String MESSAGE_TYPE_CODE_0402 = "Abgabe.ImportBestaetigen.0402";
	String AUTHOR_ID_PREFIX = "gad";
	String AUTHOR_ID_CATEGORY = "Generischer Antragsdienst";

	String READER_ID_PREFIX = "gae";
	String READER_ID_CATEGORY = "Generischer Antragsempfänger";

	String MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_240 = "urn:xoev-de:xdomea:schema:2.4.0";
	String SERVICE_XDOMEA_240 = MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_240 + "/xdomea240Antrag.wsdl";
	String MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_300 = "urn:xoev-de:xdomea:schema:3.0.0";
	String SERVICE_XDOMEA_300 = MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_300 + "/xdomea300Abgabe.wsdl";

	String BUSINESS_SCENARIO_CODE = "XDOMEAGAD_DATA";
	String BUSINESS_SCENARIO_LIST_URI = "urn:de:dataport:codeliste:business.scenario";
	String BUSINESS_SCENARIO_LIST_VERSION_ID = "1";

	@Mapping(target = "service", source = "messageTypeCode", qualifiedByName = "getServiceByMessageTypeCode")
	@Mapping(target = "businessScenarioCode", constant = BUSINESS_SCENARIO_CODE)
	@Mapping(target = "businessScenarioListUri", constant = BUSINESS_SCENARIO_LIST_URI)
	@Mapping(target = "businessScenarioListVersionId", constant = BUSINESS_SCENARIO_LIST_VERSION_ID)
	@Mapping(target = "messageTypeCode", source = "messageTypeCode", qualifiedByName = "getMessageTypeCodeByMessageTypeCode")
	@Mapping(target = "messageTypePayloadSchema", source = "messageTypeCode", qualifiedByName = "getMessageTypePayloadSchemaByMessageTypeCode")
	@Mapping(target = "authorIdentifier", source = "authorIdSuffix", qualifiedByName = "mapAuthorIdentifier")
	@Mapping(target = "readerIdentifier", source = "readerIdSuffix", qualifiedByName = "mapReaderIdentifier")
	@Mapping(target = "messageId", ignore = true)
	@Mapping(target = "messageSize", ignore = true)
	@Mapping(target = "deliveryAttributesOrigin", ignore = true)
	@Mapping(target = "deliveryAttributesDelivery", ignore = true)
	XtaMessageMetaData mapXtaMessageMetadata(XdomeaXmlValues xdomeaXmlValues);

	@Named("getServiceByMessageTypeCode")
	default String getServiceByMessageTypeCode(String messageTypeCode) {
		return switch (messageTypeCode) {
			case CODE_0201 -> SERVICE_XDOMEA_240;
			case CODE_0401, CODE_0402 -> SERVICE_XDOMEA_300;
			default -> throw new XtaClientRuntimeException("Unknown messageTypeCode: " + messageTypeCode);
		};
	}

	@Named("getMessageTypeCodeByMessageTypeCode")
	default String getMessageTypeCodeByMessageTypeCode(String messageTypeCode) {
		return switch (messageTypeCode) {
			case CODE_0201 -> MESSAGE_TYPE_CODE_0201;
			case CODE_0401 -> MESSAGE_TYPE_CODE_0401;
			case CODE_0402 -> MESSAGE_TYPE_CODE_0402;
			default -> throw new XtaClientRuntimeException("Unknown messageTypeCode: " + messageTypeCode);
		};
	}

	@Named("getMessageTypePayloadSchemaByMessageTypeCode")
	default String getMessageTypePayloadSchemaByMessageTypeCode(String messageTypeCode) {
		return switch (messageTypeCode) {
			case CODE_0201 -> MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_240;
			case CODE_0401, CODE_0402 -> MESSAGE_TYPE_PAYLOAD_SCHEMA_XDOMEA_300;
			default -> throw new XtaClientRuntimeException("Unknown messageTypeCode: " + messageTypeCode);
		};
	}

	@Named("mapAuthorIdentifier")
	@Mapping(target = "category", constant = AUTHOR_ID_CATEGORY)
	@Mapping(target = "name", ignore = true)
	@Mapping(target = "value", expression = "java( createIdentifierValue(AUTHOR_ID_PREFIX, authorIdSuffix) )")
	XtaIdentifier mapAuthorIdentifier(String authorIdSuffix);

	@Named("mapReaderIdentifier")
	@Mapping(target = "category", constant = READER_ID_CATEGORY)
	@Mapping(target = "name", ignore = true)
	@Mapping(target = "value", expression = "java( createIdentifierValue(READER_ID_PREFIX, readerIdSuffix) )")
	XtaIdentifier mapReaderIdentifier(String readerIdSuffix);

	default String createIdentifierValue(String prefix, String suffix) {
		return String.join(":", prefix, suffix);
	}
}
