package de.ozgcloud.xta.client.xdomea.reader;

import static de.ozgcloud.xta.client.xdomea.reader.XdomeaValueReader.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.xml.xpath.XPathFactory;

import lombok.Builder;
import lombok.RequiredArgsConstructor;

@Builder
@RequiredArgsConstructor
public class XdomeaValueReaderFactory {

	private final XPathFactory xPathFactory;

	XmlValueReader createXmlValueReader(String textXpathString) {
		return createXmlValueReaderFactory(textXpathString).create();
	}

	XmlValueReaderFactory createXmlValueReaderFactory(String textXpathString) {
		return XmlValueReaderFactory.builder()
				.xPathFactory(xPathFactory)
				.textXpathString(textXpathString)
				.build();
	}

	public XdomeaValueReader create() {
		return XdomeaValueReader.builder()
				.xmlValueReaders(Stream.of(
						PROCESS_ID_XPATH,
						MESSAGE_TYPE_ID_SUFFIX_XPATH,
						AUTHOR_ID_SUFFIX_XPATH,
						AUTHOR_ID_PREFIX_XPATH,
						READER_ID_SUFFIX_XPATH,
						READER_ID_PREFIX_XPATH,
						PRIMARY_DOCUMENT_NAME_XPATH
				).collect(Collectors.toMap(
						xpath -> xpath,
						this::createXmlValueReader
				)))
				.build();
	}
}
