package de.ozgcloud.xta.client;

import static de.ozgcloud.xta.client.factory.MessageMetaDataTestFactory.*;
import static de.ozgcloud.xta.client.factory.XtaClientConfigTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.xta.client.factory.XtaMessageMetaDataTestFactory;
import de.ozgcloud.xta.client.model.XtaMessage;

class FetchMessageParameterTest {

	private Consumer<XtaMessage> processMessage;

	private FetchMessageParameter parameter;

	@BeforeEach
	void setup() {
		processMessage = message -> {
		};
		parameter = new FetchMessageParameter(SELF_IDENTIFIER, processMessage, Set.of(MESSAGE_ID2));

	}

	@DisplayName("with viewed message ids from")
	@Nested
	class TestWithViewedMessageIdsFrom {

		@DisplayName("should keep client identifier")
		@Test
		void shouldKeepClientIdentifier() {
			var result = parameter.withViewedMessageIdsFrom(Collections.emptyList());

			assertThat(result.clientIdentifier()).isEqualTo(SELF_IDENTIFIER);
		}

		@DisplayName("should keep process message")
		@Test
		void shouldKeepProcessMessage() {
			var result = parameter.withViewedMessageIdsFrom(Collections.emptyList());

			assertThat(result.processMessage()).isEqualTo(processMessage);
		}

		@DisplayName("should return copy")
		@Test
		void shouldReturnCopy() {
			var result = parameter.withViewedMessageIdsFrom(Collections.emptyList());

			assertThat(result.viewedMessageIds()).containsExactly(MESSAGE_ID2);
		}

		@DisplayName("should return copy with additional viewed message ids")
		@Test
		void shouldReturnCopyWithAdditionalViewedMessageIds() {
			var result = parameter.withViewedMessageIdsFrom(List.of(XtaMessageMetaDataTestFactory.create()));

			assertThat(result.viewedMessageIds()).containsExactlyInAnyOrder(MESSAGE_ID2, MESSAGE_ID);
		}

		@DisplayName("should not change original with additional viewed message ids")
		@Test
		void shouldNotChangeOriginalWithAdditionalViewedMessageIds() {
			parameter.withViewedMessageIdsFrom(List.of(XtaMessageMetaDataTestFactory.create()));

			assertThat(parameter.viewedMessageIds()).containsExactly(MESSAGE_ID2);
		}
	}

	@DisplayName("has message already been viewed")
	@Nested
	class TestHasMessageAlreadyBeenViewed {

		@DisplayName("should return true if message id is in viewed message ids")
		@Test
		void shouldReturnTrueIfMessageIdIsInViewedMessageIds() {
			var messageMetaData = XtaMessageMetaDataTestFactory.createBuilder()
					.messageId(MESSAGE_ID2)
					.build();

			var result = parameter.hasMessageAlreadyBeenViewed(messageMetaData);

			assertThat(result).isTrue();
		}

		@DisplayName("should return false if message id is not in viewed message ids")
		@Test
		void shouldReturnFalseIfMessageIdIsNotInViewedMessageIds() {
			var messageMetaData = XtaMessageMetaDataTestFactory.createBuilder()
					.messageId(MESSAGE_ID3)
					.build();

			var result = parameter.hasMessageAlreadyBeenViewed(messageMetaData);

			assertThat(result).isFalse();
		}
	}

}