package de.ozgcloud.xta.client.extension;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import de.ozgcloud.xta.client.exception.XtaClientRuntimeException;

public record Patch(
		String filePath,
		List<PatchHunk> hunks
) {

	public static Optional<Patch> findPatchByFilePathSuffix(List<Patch> patches, String filePathSuffix) {
		return patches.stream()
				.filter(patch -> patch.filePath().endsWith(filePathSuffix))
				.findFirst();
	}

	public String apply(String content) {
		var lines = content.lines().iterator();
		var lineOffset = 0;
		var patchedContent = new StringBuilder();
		try {
			for (var hunk : hunks) {
				var patchResult = hunk.apply(lines, lineOffset);
				for (var patchLine : patchResult.lines()) {
					patchedContent.append(patchLine).append("\n");
				}
				lineOffset = patchResult.lineOffset();
			}
			lines.forEachRemaining(patchedContent::append);
		} catch (XtaClientRuntimeException exception) {
			throw new XtaClientRuntimeException("Error applying patch! filePath=" + filePath, exception);
		}
		return patchedContent.toString();
	}

	public static List<Patch> parsePatches(String patchFileContent) {
		var patches = new ArrayList<Patch>();

		var lines = patchFileContent.lines().iterator();
		while (lines.hasNext()) {
			var line = lines.next();
			if (line.startsWith("diff --git")) {
				var fileName = line.split(" ")[2];
				patches.add(new Patch(fileName, PatchHunk.parseHunks(lines)));
			}
		}
		return patches;
	}
}
