package de.ozgcloud.xta.client.extension;

import static de.ozgcloud.xta.client.extension.XtaServerSetupExtensionTestUtil.*;

import java.io.File;
import java.util.Objects;

import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.ExtensionContext;

import com.google.common.io.Files;

import de.ozgcloud.xta.client.XtaClient;
import de.ozgcloud.xta.client.XtaClientFactory;
import de.ozgcloud.xta.client.config.XtaClientConfig;
import de.ozgcloud.xta.client.core.WrappedXtaService;
import lombok.Getter;
import lombok.Setter;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

@Getter
@Setter
@Slf4j
public class XtaRemoteServerSetupExtension implements BeforeAllCallback {

	private XtaClient testClient;
	private XtaClientConfig testClientConfig;
	private XtaClient silentTestClient;
	private XtaClientConfig silentTestClientConfig;
	private XtaClient devClient;
	private XtaClientConfig devClientConfig;
	private XtaClient silentDevClient;
	private XtaClientConfig silentDevClientConfig;

	private WrappedXtaService service;
	private XtaClientFactory clientFactory;

	static final String BASE_URL = "https://li33-0005.dp.dsecurecloud.de/MB_XTA-WS/XTA210";

	@Override
	@SneakyThrows
	public void beforeAll(ExtensionContext context) {
		testClientConfig = createClientConfig(
				createClientCertKeyStore(getEnvVar("KOP_SH_KIEL_TEST_PATH"), getEnvVar("KOP_SH_KIEL_TEST_PASSWORD")),
				true);
		testClient = XtaClient.from(testClientConfig);
		silentTestClientConfig = createClientConfig(
				testClientConfig.getClientCertKeystore(),
				false);
		silentTestClient = XtaClient.from(silentTestClientConfig);

		devClientConfig = createClientConfig(
				createClientCertKeyStore(getEnvVar("KOP_SH_KIEL_DEV_PATH"), getEnvVar("KOP_SH_KIEL_DEV_PASSWORD")),
				true);
		silentDevClientConfig = createClientConfig(
				devClientConfig.getClientCertKeystore(),
				false);
		devClient = XtaClient.from(devClientConfig);
		silentDevClient = XtaClient.from(silentDevClientConfig);
	}

	private XtaClientConfig.KeyStore createClientCertKeyStore(String clientCertKeystorePath, String clientCertKeystorePassword) {
		return XtaClientConfig.KeyStore.builder()
				.content(readBytesFromFile(clientCertKeystorePath))
				.type("PKCS12")
				.password(clientCertKeystorePassword.toCharArray())
				.build();
	}

	@SneakyThrows
	XtaClientConfig createClientConfig(XtaClientConfig.KeyStore clientCertKeyStore, boolean verbose) {
		return createSpecificClientConfigBuilder()
				.logSoapRequests(verbose)
				.logSoapResponses(verbose)
				.clientCertKeystore(clientCertKeyStore)
				.build();
	}

	private String getEnvVar(String name) {
		return Objects.requireNonNull(System.getenv(name), "Environment variable " + name + " is required!");
	}

	@SneakyThrows
	private static byte[] readBytesFromFile(String path) {
		return Files.toByteArray(new File(path));
	}

	@SneakyThrows
	public XtaClientConfig.XtaClientConfigBuilder createSpecificClientConfigBuilder() {
		return createClientConfigBuilder()
				.managementServiceUrl(BASE_URL + "managementPort.svc")
				.sendServiceUrl(BASE_URL + "sendPort.svc")
				.msgBoxServiceUrl(BASE_URL + "msgBoxPort.svc");
	}

}
