package de.ozgcloud.xta.client.xdomea.reader;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

import de.ozgcloud.xta.client.exception.XtaClientRuntimeException;
import lombok.SneakyThrows;

class XmlValueReaderFactoryTest {

	private XmlValueReaderFactory factory;

	@Mock
	private XPathFactory xPathFactory;

	private final String textXpathString = "xpath/expression";

	@BeforeEach
	void setup() {
		factory = XmlValueReaderFactory.builder()
				.xPathFactory(xPathFactory)
				.textXpathString(textXpathString)
				.build();

	}

	@DisplayName("create")
	@Nested
	class TestCreate {

		@Mock
		XPath xpath;

		@Mock
		XPathExpression expression;

		@Mock
		XPathExpressionException exception;

		@BeforeEach
		void mock() {
			when(xPathFactory.newXPath()).thenReturn(xpath);
		}

		@DisplayName("should return")
		@Test
		@SneakyThrows
		void shouldReturn() {
			when(xpath.compile(textXpathString)).thenReturn(expression);

			var result = factory.create();

			assertThat(result).usingRecursiveComparison().isEqualTo(XmlValueReader.builder()
					.xPathExpression(expression)
					.build());
		}

		@DisplayName("should throw xta client runtime exception")
		@Test
		@SneakyThrows
		void shouldThrowXtaClientRuntimeException() {
			when(xpath.compile(textXpathString)).thenThrow(exception);

			assertThatThrownBy(() -> factory.create())
					.isInstanceOf(XtaClientRuntimeException.class);
		}
	}

}