package de.ozgcloud.xta.client.xdomea.reader;

import static de.ozgcloud.xta.client.factory.XmlDocumentTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.w3c.dom.Document;

public class XmlValueReaderTest {

	@DisplayName("read non empty texts")
	@Nested
	class TestReadNonEmptyTexts {
		private static final Document XML_DOCUMENT = createXMLDocument("<root><value attr=\"val\">test</value><value>test2</value></root>");

		@DisplayName("should return with two matches")
		@Test
		void shouldReturnWithTwoMatches() {
			var xmlValueReader = createXmlValueReaderWithXPath("/root/value/text()");

			var result = xmlValueReader.readNonEmptyTexts(XML_DOCUMENT);

			assertThat(result).containsExactly("test", "test2");
		}

		@DisplayName("should return with one match")
		@Test
		void shouldReturnWithOneMatch() {
			var xmlValueReader = createXmlValueReaderWithXPath("/root/value[@attr]/text()");

			var result = xmlValueReader.readNonEmptyTexts(XML_DOCUMENT);

			assertThat(result).containsExactly("test");
		}

		@DisplayName("should return with no match")
		@Test
		void shouldReturnWithNoMatch() {
			var xmlValueReader = createXmlValueReaderWithXPath("/root/missing/text()");

			var result = xmlValueReader.readNonEmptyTexts(XML_DOCUMENT);

			assertThat(result).isEmpty();
		}

	}

	private XmlValueReader createXmlValueReaderWithXPath(String xPathString) {
		return XmlValueReader.builder()
				.xPathExpression(createXPathExpression(xPathString))
				.build();
	}

}